from datetime import datetime

from sqlalchemy import Integer, DateTime, Column, String
from sqlalchemy.ext.declarative import declared_attr

from .entity_base import EntityBase


class Entity(EntityBase):
    Id = Column(
        Integer,
        primary_key=True
    )

    @declared_attr
    def CreatedByUserId(cls):
        return Column(Integer, index=False, unique=False, nullable=False, default=0)

    @declared_attr
    def CreationDate(cls):
        return Column(DateTime, index=False, unique=False, nullable=False, default=datetime.now)

    @declared_attr
    def LastUpdatedUserId(cls):
        return Column(Integer, index=False, unique=False, nullable=True)

    @declared_attr
    def LastUpdatedDate(cls):
        return Column(DateTime, index=False, unique=False, nullable=True)

    @declared_attr
    def IsDeleted(cls):
        return Column(Integer, index=False, unique=False, nullable=False, default=0)

    @declared_attr
    def Comments(cls):
        return Column(String(1000), index=False, unique=False, nullable=True)

    # @declared_attr
    # def RowVersion(cls):
    #     return Column(TIMESTAMP(), default=text('DEFAULT'))



from datetime import datetime


class EntityBase:

    def __init__(self,
                 Id: int = None,
                 CreatedByUserId: int = None,
                 CreationDate: datetime = None,
                 LastUpdatedUserId: int = None,
                 LastUpdatedDate: datetime = None,
                 IsDeleted: bool = None,
                 Comments: str = None,
                 # RowVersion: bytes = None,
                 *args, **kwargs
                 ):
        super().__init__(*args, **kwargs)
        self.Id = Id
        self.CreatedByUserId: int = CreatedByUserId
        self.CreationDate: datetime = CreationDate
        self.LastUpdatedUserId: int = LastUpdatedUserId
        self.LastUpdatedDate: datetime = LastUpdatedDate
        self.IsDeleted: bool = IsDeleted
        self.Comments: str = Comments
        # self.RowVersion: bytes = RowVersion



import os
from typing import List, Type, TypeVar

import yaml

from .models.base.base_config import BaseConfig
from ..utils import Utils, ModuleFinder

T = TypeVar('T')


class ConfigManager:
    def __init__(self, root_directory: str, module_finder: ModuleFinder) -> None:
        # Create an empty list with items of type T
        self.module_finder = module_finder
        self.configs = self.__get_configs(root_directory)

    def get_all(self):
        return self.configs

    def get(self, generic_type: Type[T]) -> T:
        for config in self.configs:
            config_type = config.get("type")
            if isinstance(config.get("instance"),
                          generic_type) or config_type is generic_type or generic_type.__module__ == config_type.__module__:
                return config.get("instance")

    def set(self, generic_type, instance_property, property_value):
        config_instance = self.get(generic_type=generic_type)
        if config_instance is None:
            config_instance = generic_type()
            self.configs.append({"type": generic_type, "instance": config_instance})
        setattr(config_instance, instance_property, property_value)

    def __get_configs(self, root_directory: str) -> List[dict]:
        self.module_finder.import_modules_by_name_ends_with(name='Configs')

        environment = os.getenv('PYTHON_ENVIRONMENT', None)
        config_name = "application.yml"
        if environment is not None and environment != '':
            config_name_split = "application.yml".split('.')
            config_name = f'{config_name_split[0]}.{environment}.{config_name_split[1]}'
        config_path = os.path.join(root_directory, config_name)
        configs: List[dict] = []
        if os.path.exists(config_path):
            with open(config_path, 'r') as yml_file:
                loaded_configs = yaml.load(yml_file, Loader=yaml.FullLoader)
            for config in BaseConfig.__subclasses__():
                config_instance = config()
                class_name = Utils.get_config_name(
                    config_instance.__class__.__name__)
                class_properties = [a for a in dir(
                    config_instance) if not (a.startswith('_'))]
                for prop in class_properties:
                    property_name = prop.upper()
                    if class_name in loaded_configs:
                        loaded_config = loaded_configs[class_name]
                        if property_name in loaded_config:
                            config_value = loaded_config[property_name]
                        elif property_name == 'ROOT_DIRECTORY':
                            config_value = root_directory
                        else:
                            config_value = None
                    else:
                        if property_name == 'ROOT_DIRECTORY':
                            config_value = root_directory
                        config_value = None

                    environment_name = f'{class_name}_{property_name}'
                    property_value = os.getenv(environment_name, config_value)
                    setattr(config_instance, prop, property_value)
                configs.append({"type": config, "instance": config_instance})
        return configs



from os import getenv
from pathlib import Path

from setuptools import setup, find_packages

here = Path(__file__).parent.resolve()
long_description = (here / 'README.md').read_text(encoding='utf-8')

env_version = getenv('PYPI_PACKAGE_VERSION', default='0.5.12')
version = env_version.replace('v', '')
setup(
    name='pdip',
    version=f'{version}',
    description='Python Data Integrator infrastructures package',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/ahmetcagriakca/pdip',
    download_url=f'https://github.com/ahmetcagriakca/pdip/archive/refs/tags/v{version}.tar.gz',
    author='Ahmet Çağrı AKCA',
    author_email='ahmetcagriakca@gmail.com',
    license='MIT',
    packages=find_packages(exclude=["tests", "tests*", "test_*", '__pycache__', '*.__pycache__', '__pycache.*', '*.__pycache__.*']),
    zip_safe=False,
    keywords=['PDI', 'API', 'ETL', 'PROCESS', 'MULTIPROCESS', 'IO', 'CQRS', 'MSSQL', 'ORACLE', 'POSTGRES', 'MYSQL', 'CSV'],
    python_requires='>=3.6, <3.10',
    install_requires=[
        "coverage",
        "cryptography",
        "dataclasses",
        "jsonpickle",
        "Fernet",
        "func-timeout",
        "Flask",
        "Flask_Cors",
        "Flask-Ext",
        "Flask-Injector",
        "flask-restx",
        "injector",
        "pandas",
        "PyYAML",
        "requests",
        "rpyc",
        "SQLAlchemy",
        "Werkzeug"
    ],
    extras_require={
        "integrator" : ["cx_Oracle==8.2.1","kafka-python==2.0.2","mysql-connector-python==8.0.26","psycopg2-binary==2.8.6","pyodbc==4.0.30"],
        "preferred" : [
        "coverage==5.5",
        "cryptography==3.3.2",
        "dataclasses==0.6",
        "jsonpickle==2.0.0",
        "Fernet==1.0.1",
        "func-timeout==4.3.5",
        "Flask==1.1.4",
        "Flask_Cors==3.0.10",
        "Flask-Ext==0.1",
        "Flask-Injector==0.12.3",
        "flask-restx==0.5.1",
        "injector==0.18.4",
        "pandas==1.1.5",
        "PyYAML==5.4",
        "requests==2.26.0",
        "rpyc==5.0.1",
        "SQLAlchemy==1.4.19",
        "Werkzeug==1.0.1"
        ]
    },
    classifiers=[
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
    project_urls={  # Optional
        'Bug Reports': 'https://github.com/ahmetcagriakca/pdip/issues',
        'Source': 'https://github.com/ahmetcagriakca/pdip',
    },
)
